package server

// REFACTOR: APP DEDICATED CODE
import (
	"time"

	"git.pbiernat.dev/egommerce/api-entities/http"
	"git.pbiernat.dev/egommerce/basket-service/internal/service"
	"git.pbiernat.dev/egommerce/basket-service/internal/ui"
	"github.com/gofiber/fiber/v2"
)

func (s *Server) GetBasketHandler(c *fiber.Ctx) error {
	req := new(http.GetBasketRequest)
	if err := c.BodyParser(req); err != nil {
		return s.Base.Error(c, 400, err.Error())
	}

	basketID := req.BasketID
	basketSrv := service.NewBasketService(s.Database, s.Cache, s.Eventbus, s.Logger)
	basket, err := basketSrv.FetchFromDB(c.Context(), basketID)
	if err != nil {
		return s.Base.Error(c, 400, "Failed to retrieve basket")
	}

	res := &http.GetBasketResponse{
		ID:        basket.ID,
		State:     basket.State,
		CreatedAt: time.Duration(basket.CreatedAt.Time.Unix()),
	}
	if basket.UpdatedAt.Time.Unix() > 0 {
		res.UpdatedAt = time.Duration(basket.UpdatedAt.Time.Unix())
	}

	return c.JSON(res)
}

func (s *Server) GetBasketItemsHandler(c *fiber.Ctx) error {
	basketID := c.Params("basketId", "")
	basketSrv := service.NewBasketService(s.Database, s.Cache, s.Eventbus, s.Logger)
	items, err := basketSrv.FetchItems(c.Context(), basketID)
	if err != nil {
		return s.Base.Error(c, 400, "Failed to retrieve basket")
	}

	var res []*http.GetBasketItemsResponse
	for _, item := range items {
		resItem := &http.GetBasketItemsResponse{
			ID:        item.ID,
			BasketID:  item.BasketID,
			ProductID: item.ProductID,
			Quantity:  item.Quantity,
			Price:     item.Price,
			CreatedAt: time.Duration(item.CreatedAt.Time.Unix()),
		}
		if item.UpdatedAt.Time.Unix() > 0 {
			resItem.UpdatedAt = time.Duration(item.UpdatedAt.Time.Unix())
		}
		res = append(res, resItem)
	}

	return c.JSON(res)
}

func (s *Server) CheckoutHandler(c *fiber.Ctx) error {
	reqID, _ := s.Base.GetRequestID(c)
	req := new(http.BasketCheckoutRequest)
	if err := c.BodyParser(req); err != nil {
		return s.Base.Error(c, 400, err.Error())
	}

	basketID := req.BasketID
	basketSrv := service.NewBasketService(s.Database, s.Cache, s.Eventbus, s.Logger)
	res, err := ui.CheckoutBasket(basketSrv, basketID, reqID)
	if err != nil {
		return s.Base.Error(c, 400, "Failed to create order")
	}

	return c.JSON(res)
}
